﻿using System;
using System.Net;
using System.Net.NetworkInformation;
using System.Net.Sockets;
using System.ServiceModel;
using System.Threading;
using DistributedGA.Core.Domain;
using DistributedGA.Core.Interface;

namespace DistributedGA.Core.Implementation {
  public class WcfMessageService : IMessageService, IDisposable {
    public event EventHandler<MessageRecieveEventArgs> OnDataRecieved;

    private ManualResetEvent _ResetEvent = new ManualResetEvent(false);

    private IMessageContract messageContract = null;

    public int Init(string ip) {
      int port = 0;
      port = FreeTcpPort();

      messageContract = new MessageContractImpl();
      messageContract.MessageRecieved += new EventHandler<MessageRecieveEventArgs>(OnMessageRecieved);

      var serviceUrl = "DistributedGA.svc";
      new Thread(() => {
        try {
          var baseUri = new Uri(string.Concat("net.tcp://", ip, ":", port, "/DistributedGA"));
          var serviceUri = new Uri(baseUri, serviceUrl);
          NetTcpBinding binding = new NetTcpBinding();
          binding.MaxReceivedMessageSize = 20000000;
          binding.MaxBufferSize = 20000000;
          binding.MaxBufferPoolSize = 20000000;
          binding.ReaderQuotas.MaxArrayLength = 20000000;
          binding.ReaderQuotas.MaxDepth = 32;
          binding.ReaderQuotas.MaxStringContentLength = 20000000;

          using (var host = new ServiceHost(messageContract, serviceUri)) {
            host.AddServiceEndpoint(typeof(IMessageContract), binding, serviceUri);

            host.Open();

            _ResetEvent.WaitOne();
          }
        }
        catch (Exception) {  }
      }).Start();
      return port;

    }

    public void Dispose() {
      _ResetEvent.Set();
      _ResetEvent.Dispose();
    }

    private bool IsPortAvaiable(int port) {
      //int port = 456; //<--- This is your value
      bool isAvailable = true;

      // Evaluate current system tcp connections. This is the same information provided
      // by the netstat command line application, just in .Net strongly-typed object
      // form.  We will look through the list, and if our port we would like to use
      // in our TcpClient is occupied, we will set isAvailable to false.
      IPGlobalProperties ipGlobalProperties = IPGlobalProperties.GetIPGlobalProperties();
      TcpConnectionInformation[] tcpConnInfoArray = ipGlobalProperties.GetActiveTcpConnections();
      foreach (TcpConnectionInformation tcpi in tcpConnInfoArray) {
        if (tcpi.LocalEndPoint.Port == port) {
          isAvailable = false;
          break;
        }
      }
      foreach (var item in ipGlobalProperties.GetActiveTcpListeners()) {
        if (item.Port == port) {
          isAvailable = false;
          break;
        }
      }
      return isAvailable;
    }

    private int FreeTcpPort() {
      for (int i = 11000; i <= 11100; i++) {
        if (IsPortAvaiable(i))
          return i;
      }
      return 0;
    }

    //private int FreeTcpPort() {
    //  TcpListener l = new TcpListener(IPAddress.Loopback, 0);
    //  l.Start();
    //  int port = ((IPEndPoint)l.LocalEndpoint).Port;
    //  l.Stop();
    //  return port;
    //}

    private void OnMessageRecieved(object sender, MessageRecieveEventArgs e) {
      if (OnDataRecieved != null) {
        OnDataRecieved(this, e);
      }
    }
  }
}
